<?xml version="1.0"?>
<!DOCTYPE methoddef SYSTEM "rpc-method.dtd">
<!--
    Generated automatically by make_method v1.09, Sat Sep  4 08:46:54 2004

    Any changes made here will be lost.
-->
<methoddef>
<name>get_reports</name>
<version>1.0</version>
<signature>array array</signature>
<help>
Returns an hash which contains two arrays of all call log data
</help>
<code language="perl">
<![CDATA[
#!/opt/perl5/bin/perl
###############################################################################
#
#   Sub Name:       grab_reports
#
#   Description:    Grabs all call records
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#
#   Globals:        None.
#
#   Environment:    None.
#
#   Returns:        Hash with two arrays
#
###############################################################################
sub get_reports
{
open(LOG, ">>/tmp/xpl.log"); print LOG localtime() . ": $$: reporting.xpl(@_)\n"; close(LOG);
	my ($srv, $req) = @_;
	my ($sid) = shift(@{$req});
	my ($uae) = shift(@{$req});

	use strict;
	use Date::Format;
	use File::Copy;

	open(LOGGER, ">>/tmp/log_pulling.log") or die "couldn't open a log file";

	print LOGGER time() . ": Starting the xpl portion of logging\n";

	my $reports;
	
	# make a couple of dir's we need
        mkdir('/var/log/asterisk/real/');
        mkdir('/var/log/asterisk/nodata/');

	# list of file and locations to check for logs
	my $cdr_dir = '/var/log/asterisk/cdr-csv/';
	my $cdr_file = 'Master.csv';

	my $queue_dir = '/var/log/asterisk/';
	my $queue_file = 'queue_log';

	# backup directory
	my $backup_directory;

	# handle all cdr, or non queue calls
	my $queued_cdr_reports = check_for_queued_logs($cdr_dir);
	if ($queued_cdr_reports)
	{
		# We're here if there was a big log file that needed to be split up
		# we enter here for each file with *queued* in the name
		$reports->{'regular_reports'} = grab_regular_reports($queued_cdr_reports);
		
		$queued_cdr_reports =~ m|([^/]+)$|;

		unless(rename($queued_cdr_reports, "/var/log/asterisk/real/$1"))
		{
			$reports->{'regular_reports'} = undef;
			print LOGGER "Unable to rename file $queued_cdr_reports /var/log/asterisk/real/$1 :  $! $@\n"; 
		}
	}
	else
	{
		# no queued files, let's look at the origional file
		my $call_logs = get_new_logs($cdr_dir, $cdr_file);
		if ($call_logs)
		{
			$reports->{'regular_reports'} = grab_regular_reports($call_logs);

			$call_logs =~ m|([^/]+)$|;

			unless(rename($call_logs, "/var/log/asterisk/real/$1"))
			{
				$reports->{'regular_reports'} = undef;
				print LOGGER "Unable to rename file $call_logs /var/log/asterisk/real/$1 :  $! $@\n";
			}
		}
	}

	#handle all queue calls
	my $queued_queue_reports = check_for_queued_logs($queue_dir);
	if ($queued_queue_reports)
	{
		$reports->{'queue_reports'} = grab_queue_reports($queued_queue_reports);

		$queued_queue_reports =~ m|([^/]+)$|;

		unless(rename($queued_queue_reports, "/var/log/asterisk/real/$1"))
		{
			$reports->{'queue_reports'} = undef;
			print LOGGER "Unable to rename file $queued_queue_reports /var/log/asterisk/real/$1 :  $! $@\n";
		}
	}
	else
	{
		my $call_logs = get_new_logs($queue_dir, $queue_file);
		if ($call_logs)
		{
			$reports->{'queue_reports'} = grab_queue_reports($call_logs);

			$call_logs =~ m|([^/]+)$|;

			unless(rename($call_logs, "/var/log/asterisk/real/$1"))
			{
				$reports->{'queue_reports'} = undef;
				print LOGGER "Unable to rename file $call_logs /var/log/asterisk/real/$1 :  $! $@\n";
			}
		}
	}

	print LOGGER time() . ": Stopping the xpl portion of logging\n";

	# if this is a Unified Agent Edition server, 
	# grab call recordings for directory and add to the output variable as an array
	if($uae)
	{
		print LOGGER time() . ": Unified Agent Edition : Starting call file importing\n";
		
		my $path = "/var/spool/asterisk/monitor";
		opendir(DIR, $path) or die "Couldn't open $path: $!";
		
		my @calls = grep {!/^\.\.?$/} readdir(DIR);
		
		$reports->{'calls'} = \@calls;
		
		print LOGGER time(). ": Unified Agent Edition : Stoppoing call file importing\n";
		closedir(DIR);
	}

	close(LOGGER);
	return $reports;

	# just as it sounds, look in a given $dir for filed with queued in the file name
	sub check_for_queued_logs
	{
		my $dir = shift(@_);
		
		opendir(DIR, "$dir");
		my @files = readdir(DIR);
		closedir(DIR);

		foreach my $file (@files)
		{
			if ($file =~ /queued/)
			{
				return "$dir$file";
			}
			elsif ($file =~ /queue_log\.\d{1,}/)
			{
				return "$dir$file";
			}
		}
		return undef;
	}

	sub grab_regular_reports
	{
		my $file = shift(@_);

		if (-e "$file")
		{
			my $cleaned_up_reports;
			my $time = time;

			my $devices = devices_list();

			open(CDR, "<$file") || die "Can't open csv: $!";
			while (<CDR>)
			{
				# strip out nasty characters
				s/[^\x20-\x7e]//g;

				# strip out excess quotes
				s/""//g;

				# append a comma to our string for faster parsing
				$_ .= ",";

				# need to empty out our @fields array
				my @fields = ();

				# read each element into an array
				s/(?:"(.*?)",|([^,]*),)/push @fields, $+/eg;

				# remove the 11th element of the array
				splice(@fields, 10, 1);

			        push(@fields, $fields[1]);

				# replace the sip/<mac>-<random garbage>
				# with the extension number
				if (length($fields[1]) > 6)
				{
					my ($mac) = ($fields[5] =~ /^"?([^-]+)/);
					if ($devices->{$mac})
					{
						$fields[1] = "$devices->{$mac}";
					}
				}


				$cleaned_up_reports .= sprintf("\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",%d,%d,\"%s\",%d\n", @fields);
			}
			close(CDR);
			
			return $cleaned_up_reports;
		}
	}

	sub grab_queue_reports
	{
		my $file = shift(@_);
		my @enter;
		my @abandon;
		my @callconnect;

		my $reports;
		
		if (-e "$file")
		{
			my @enter;
			my @abandon;
			my @callconnect;

			open(LOG,"$file");
			while(my $line = <LOG>) 
			{
				chomp($line);
				if ($line =~ /(\d+)\|(.+?)\|(.+?)\|NONE\|ENTERQUEUE\|(.*?)\|(.*)/) 
				{
					push(@enter, qq{","$2","$3","} . ($4 ? "$4" : "NULL") . qq{","} . ($5 ? "$5" : "NULL") . qq{","} . time2str("%Y-%m-%d %H:%M:%S", $1) . q{"});
				} 
				elsif ($line =~ /(\d+)\|(.+?)\|(.+?)\|NONE\|ABANDON\|(\d*?)\|(\d*?)\|(\d*)/) 
				{
					push(@abandon, qq{","$2","$3","} . ($4 || 0) . qq{","} . ($5 || 0) . qq{","} . ($6 || 0) . qq{","} . time2str("%Y-%m-%d %H:%M:%S", $1) . qq{"});
				} 
				elsif ($line =~ /(\d+)\|(.+?)\|(.+?)\|(.*?)\|AGENTDUMP\|/) 
				{
					push(@callconnect, qq{","$2","$3","$4","agentdump","NULL","NULL","NULL","} . time2str("%Y-%m-%d %H:%M:%S", $1) . qq{"});
				} 
				elsif ($line =~ /(\d+)\|(.+?)\|(.+?)\|(.*?)\|CONNECT\|(\d+)/) 
				{
					push(@callconnect, qq{","$2","$3","$4","connect","$5","NULL","NULL","} . time2str("%Y-%m-%d %H:%M:%S", $1) . q{"});
				} 
				elsif ($line =~ /(\d+)\|(.+?)\|(.+?)\|(.*?)\|COMPLETEAGENT\|(\d+)\|(\d+)[|]{0,1}(\d*)/) 
				{
					push(@callconnect, qq{","$2","$3","$4","completeagent","$5","$6","} . ($7 ? "$7" : "NULL") . q{","} . time2str("%Y-%m-%d %H:%M:%S", $1) . q{"});
				} 
				elsif ($line =~ /(\d+)\|(.+?)\|(.+?)\|(.*?)\|COMPLETECALLER\|(\d+)\|(\d+)[|]{0,1}(\d*)/) 
				{
					push(@callconnect, qq{","$2","$3","$4","completecaller","$5","$6","} . ($7 ? "$7" : "NULL") . qq{","} . time2str("%Y-%m-%d %H:%M:%S", $1) . qq{"});
				} 			
			}
			
			close(LOG);

			if (scalar(@enter) == 0 && scalar(@abandon) == 0 && scalar(@callconnect) == 0)
			{
				$file =~ /(.*)\/.*\.(.*)$/;
				my $file_to_unlink = $1 . "/" . $backup_directory . "/queue_log." . $2;
				rename($file_to_unlink, "/var/log/asterisk/nodata/queue_log.$2");
			}
			else
			{
				if (scalar(@enter))
				{
					$reports->{'enter'} = \@enter;
				}

				if (scalar(@abandon))
				{
					$reports->{'abandon'} = \@abandon;
				}

				if (scalar(@callconnect))
				{
					$reports->{'callconnect'} = \@callconnect;
				}
			}
			
			return $reports;
		}
	}

	sub devices_list
	{
		my %devices = ();

		my @files = ('/etc/asterisk/fonality/internal.conf', '/etc/asterisk/fonality/external.conf');

		foreach my $file (@files)
		{
		    open(FILE, "<$file");
		    for my $line (<FILE>)
		    {
			if ($line =~ /\s*exten\s*=>\s*(.*),.*,Dial\(((?:SIP|Zap|IAX2)[^,]+),/)
			{
			    $devices{$2} = $1;
			}
		    }
		    close(FILE);
		}

		return \%devices;
	}

	sub get_new_logs
	{
		my $path = shift(@_);
		my $file = shift(@_);

		my $origional_file = "$path$file";

		my $new_file = $origional_file . ".queued." . time();

		# used to place log files in a directory on a week by week basis
		my @time = localtime();
		$backup_directory = sprintf ("%02d-%02d-%02d", substr($time[5] + 1900, 2), $time[4] + 1, int(($time[3] - 1) / 7) * 7 + 1);
		mkdir("$path$backup_directory"); 

		if (-e $origional_file)
		{
			# need to move the file out of the way and get asterisk going again
			rename($origional_file, $new_file);

			# lets restart the asterisk logger just to make sure we're good to go with
			# opening a new log file
			 my $output = `/usr/sbin/asterisk -r -x 'logger reload'`;
			print LOGGER "Output: $output\n"; 
			
			# find out how big this file is
			my $size = (stat($new_file))[7];

			# if the file passed in is smaller then .8MB then this file is good enough
			if ($size <= 800000)
			{
				my $destination_file = $origional_file . ".queued." . time();

				# The resultant file is returned to be processed, it's renamed with .queued. in case anything goes wrong
				rename($new_file, $destination_file);

				# The resultant file is uses as a copy to remain on the pbxtra/trixbox


				copy($destination_file, "$path$backup_directory/$file." . time());
				return $destination_file;
			}
			else
			{
				# that pesky file is just too darn big, let's go ahead and
				# break it down into smaller pieces, and return just one piece

				my $file_ending = 1;
				my $size;

				open(FILE, "<$new_file");
				
				$origional_file =~ /(.*\/)(.*)/;
				my $tmpfile = $1 . "$2.csv.queued.";

				open(TEMPFILE, ">$tmpfile$file_ending") or die print "Problems: $!\n";

				# as long as there's data i the file
				while (<FILE>)
				{
					my $line = $_;
						
					# clean it up
					chomp($line);
					print TEMPFILE "$line\n";

					# record the size
					$size += length($line);
			
					# file too big? let's open a new one!
					if ($size >= (800000))
					{
						close(TEMPFILE);
						$file_ending++;
						open(TEMPFILE, ">$tmpfile$file_ending");
						$size = 0;
					}
				}
				close(TEMPFILE);
				close(FILE);

				# this file remains on the pbxtra/trixbox for history
				my $destination_file = "$path$backup_directory/$file." . time();

				rename($new_file, $destination_file);
				
				print LOGGER "Returning: $tmpfile$file_ending\n";

				# we return the last queued file put together
				return "$tmpfile$file_ending";
			}
		}
	}
}

__END__
]]></code>
</methoddef>
